# Created by Hugo Cruz Jimenez, August 2011, KAUST
from NormSample import *
import math
import numpy as np
def CalcSigfromD(*args):
#function [sigma,dmax,CSseed] = CalcSigfromD(D,mech,CSseed);
#
#  function [sigma,dmax,Rseed] = CalcSigfromD(D,'mech',CSseed) 
#  calculates the slip variation around the mean. The scaling of
#  displacement variability depends on moment and/or average 
#  displacement where the latter provides higher correlation, 
#  and hence a better way for prediction.
#  The actual value is picked from a normal distribution 
#  within the error bounds of measured sig. vs. Mo.
#  The same method is applied to compute a max. value for
#  the displacement.
#
#  See also NORMSAMPLE
#
#  input: D      - mean slip (in cm)
#    mech    - 'ss' or 'SS' for strike-slip events; 
#             'ds' or 'DS' for dip-slip events
#             'al' or 'AL' for all events
#    CSseed - optional: 2x2-array of seed values for random
#       number generator
#       if given as 'off', the sample is given as computed,
#       without randomization (ignoring aleatory variability)
#
#  output: sigma  - slip variation on fault plane (in cm)
#     dmax   - max. displacement on fault plane (in cm)
#     CSseed - 2x2-array of seeds for the random number generator

#  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
#  06/26/97
#  last change 11/08/99
#  ------------------------------------------------

    if len(args) == 2:
        D=args[0]; mech=args[1]; CSseed = {} 
    else:
        D=args[0]; mech=args[1]; CSseed = args[2]
    #%% constants derived from regression analysis

    #%% for slip variation
    if mech == 'al' or mech == 'AL':
        ads = 0.809;    bds = 0.2942;  vards = 0.14
    elif mech == 'ss' or mech == 'SS':
        ads = 0.687;    bds = 0.516;   vards = 0.14
    elif mech == 'ds' or mech == 'DS':
        ads = 0.969;    bds = -0.123;  vards = 0.12
                   
    #%% for max. displacement
    if mech == 'al' or mech == 'AL':
        mds = 0.783;    xds = 1.024;  vrds = 0.16
    elif mech == 'ss' or mech == 'SS':
        mds = 0.677;    xds = 1.2078; vrds = 0.16
    elif mech == 'ds' or mech == 'DS':
        mds = 0.969;    xds = 0.684;  vrds = 0.15

    #%% compute central values for variation and max. displacement
    logsig = ads*math.log10(D) + bds
    logmax = mds*math.log10(D) + xds

    #%% pick sigma value from normal distribution
    if len(CSseed) == 0:
        CSseed = np.zeros((2,2))
        sigma,CSseed[0,] = NormSample(logsig,0.5*vards,100)
        dmax,CSseed[1,]  = NormSample(logmax,0.5*vrds,100)
 
    elif len(CSseed) != 0:
        if type(CSseed) == str:
            if CSseed == 'off':
                sigma=logsig
                dmax = logmax
            else: print '*** ERROR *** No valid CSseed-value in CalcSigfromD ****'
            return
  
        else:
            sigma,CSseed[0,] = NormSample(logsig,0.5*vards,100,CSseed[0,])
            dmax,CSseed[1,]  = NormSample(logmax,0.5*vrds,100,CSseed[1,])
    
        sigma = 10**sigma
        dmax = 10**dmax


    return sigma,dmax,CSseed
