# Created by Hugo Cruz Jimenez, August 2011, KAUST
import sys
import numpy as np
import math as ma
from fvecprint52 import *

def SXSLIPinput2C(*args):

    #function SXSLIPinput2C(SS,DS,time,rise,rpar,oldmod) 
    #
    #  SXSLIPinput2C(SS,Ds,time,rise,rpar,'oldmod')
    #
    #  creates a formatted input file to be used in within the
    #  COMPSYN package, subroutine SLIPDT which performs the
    #  integration of the tractions on the fault plane dotted
    #  into the slip distribution. This function reads the
    #  slip, rupture time and rise time distributions as well
    #  as some general source parameters, and generates the
    #  input files to run SLIPDT and the subsequent SEESLO
    #  programs to calculate the seismograms.
    #  NOTE that some SLIPDT-input values are hard-wired (i.e.
    #  'decay','rnpw','numin','nvmin','durfac')
    #
    #  INPUT: SS,DS - matrices of strike-slip and dip-slip values;
    #                 note: LL strike-slip, all amps POSITIVE
    #                       RL strike-slip, all amps NEGATIVE
    #                       +y DOWN for dip-slip POSITIVE
    #                       +y UP   for dip-slip NEGATIVE
    #	  time  - matrix of rupture times
    #	  rise  - matrix of rise times
    #	  rpar	- structural array that contains all vital
    #		  parameters for DWFE/ISO-synthetics
    #	  oldmod- string for model name used in previous calculations
    #
    #  OUTPUT:	- written to disk as *SLD-files for the slip model
    #		  and executables for DWFE code
    #			
    #  Martin Mai, March 2003
    #  ----------------------
    
    #%% in case you want to keep all output generated by Xolson, Xnoslo
    #%% Xtsurf, Xslip, Xseis, set this variable to 'n'; if you want to
    #%% delete only the generated post-script files, set it to 'ps'
    
    SS=args[0]; DS=args[1]; time=args[2]; rise=args[3]; rpar=args[4]; oldmod=args[5]
        
    ddel = 'all'
    
    #%% setting up header with general input parameters 
    #%% default values
    #rpar.Mo = fmomentN(slip,rpar.dim);
    model = rpar['model']
    vrat  = rpar['vrat']	 			#% vr/vs-ratio
    dip   = rpar['dip']				#% dip angle	
    dur   = rpar['dur']				#% duration (ave. rise time)
    rake  = rpar['rake']				#% rake angle
    MoD   = rpar['Mo']*1.0e7				#% moment in dyne-cm
    fq    = rpar['freq']				#% frequency range
    scfc  = rpar['STF']				#% slip-velocity function
    samp  = rpar['samp']
    htop  = rpar['htop']
    hdip  = htop/ma.sin(np.pi*dip/180.0)        #% extension of dipping fault
#    hdip  = htop/(ma.sin(np.pi*float(dip)/180.0))        #% extension of dipping fault
                                         #% towards the surface
    xmin  = rpar['xmin']
    
    #%% distance vectors down-dip, along-strike, 
    asc   = rpar['lx']+xmin
#   asc   = rpar['lx']+xmin
    asc2   = rpar['lx']
    print 'xmin', xmin
    print 'asc2', asc2
    print 'asc',asc
#    print 'This is lx', asc2
    ddc   = rpar['lz']+hdip 
    
    
    #% fix integration boundaries%% slip on depth-lines in vp/vs-profile
#    umin  = min(asc)+0.01
#    umax  = max(asc)-0.51    #% + 0.5*samp(2);  
#    vmin  = min(ddc)+0.01    #% - 0.5*samp(1); 
#    vmax  = max(ddc)-0.51    #% + 0.5*samp(2);			
    umin  = min(asc)
    umax  = max(asc)    #% + 0.5*samp(2);  
    vmin  = min(ddc)    #% - 0.5*samp(1); 
    vmax  = max(ddc)   #% + 0.5*samp(2);			
#    vmax  = max(ddc)-0.01    #% + 0.5*samp(2);			
    if vmin < 0: vmin = 0 
    
    #%% default settings for integration, to be used in SLIPDT-code 
    decay= float(sys.argv[36])		#% power decay
    xobs = float(sys.argv[37])   	#% observer locations, 
    yobs = float(sys.argv[38])		#% give [0 0] for all
    rnpw = float(sys.argv[39])                 #% # of points in quadrature
    numin= float(sys.argv[40])
    nvmin= float(sys.argv[41])          	#% min # of points in x,y
    durfc= float(sys.argv[42])          #%  duration fac for rise time
    vrat1 = float(sys.argv[43])                        #% standard for NO change
                                        #% in vr, vrat
#xobs,yobs,rnpw,numin,nvmin
    
    #%% convert slip distribution to dip-slip or strike-slip velocities
    
    #%% RAKE CONVENTION AND SET-UP IN DWFE/ISOCHRONE CODES:
    #%% rake =   0 --> left-lateral SS
    #%% rake = 180 --> right-lateral SS
    #%% rake =  90 --> reverse faulting
    #%% rake = 270 --> normal faulting
    #%% in DWFE/ISOCHRONE-codes the convention is
    #%% for SS, pos. amplitudes --> left-lateral motion
    #%%	    neg. amplitudes --> right-lateral motion
    #%% for DS, pos. amplitudes --> +Y-side moves down-dip
    #%%  	    neg. amplitudes --> +Y-side moves up-dip		
    
    #rak2 = (pi/180)*rake;
    #ds = slip.*sin(-rak2);			% vertical slip = dip slip
    #ss = slip.*cos(rak2); 	    		% horizontal slip = strike slip
    #%% it is assumed that DS motions are given for thrust-faulting events
    #%% as positive slip, and hence this needs to be adjusted ....
    DS = (-1)*DS
    sv = SS/rise				# strike-slip velocity amplitude
    print 'formas de SS, DS y rise',np.shape(SS),np.shape(DS),np.shape(rise)
    dv = DS/rise				# dip-slip velocity amplitude
    
    
    #%% open output files, and write headers
    outfile = '{0}{1}'.format(model,'.SLD')
#6   ff2 = '{0}{1}'.format('run.SS.',model)
    
    f1 = open(outfile,'w')
#6    f2 = open(ff2,'w')
    
#    f1.write( 'title: {0:8}, {1:8}\n'.format(model, oldmod))
    f1.write( 'title: {0:12}, {1:12}\n'.format(model, oldmod))
#    print 'title: {0:8}, {1:8}\n'.format(model, oldmod)
    f1.write( 'f1, f2, f3: {0:3.1f}, {1:3.1f}, {2:3.1f}\n'.format(fq[0],fq[1],fq[2]))
    f1.write( 'power decay: {0:3.1f}\n'.format(decay))
    f1.write( 'source time function (Box, Exp, Kos): {0}\n'.format(scfc))
    f1.write( 'umin, umax for integral on fault plane: {0:5.2f}, {1:5.2f}\n'.format(umin,umax))
    f1.write( 'vmin, vmax for integral on fault plane: {0:5.2f}, {1:5.2f}\n'.format(vmin,vmax))
    f1.write( 'desired rupture/shear velocity ratio: {0:4.2f}\n'.format(vrat1))
    f1.write( '==========  OBSERVER INFORMATION  ==========\n')
    f1.write( '   X   Y   pts/wvln   min U samp   min Vsamp\n')
    f1.write( '   -   -   --------   ----------   ---------\n')
#    f1.write( '{0:3.1f}, {1:3.1f},{2},{3},{4}\n'.format(xobs,yobs,rnpw,numin,nvmin))
    f1.write( '{0:3.1f}, {1:3.1f},{2:5.1f},{3:10.0f},{4:10.0f}\n'.format(xobs,yobs,rnpw,numin,nvmin))
#    f1.write( '{0:3.1f}, {1:3.1f},%{2:5.1f},{3:10.1f},{4:10.1f}\n'.format(xobs,yobs,rnpw,numin,nvmin)
    f1.write( '==============   SLIPMODEL   ===============\n')
    f1.write( 'duration factor: {0:4.1f}\n'.format(durfc))
    f1.write( 'rup time based on rup/shear vel ratio: {0:4.2f}\n'.format(vrat1))
    f1.write( 'desired moment (cr=what model below gives): {0:5.2e}\n'.format(MoD))
    
    
###    for i = 1:size(time,1);
    #%% write slip velocity and amplitudes into output files
    cc1,cc2=np.shape(time)
    print 'cc1', cc1
    for i in range(0,cc1):
#uf    for i in range(0,cc1):
        f1.write('------ down dip coordinate, v = {0:5.3f}\n'.format(ddc[i]))
        fvecprint52('uc:',asc,f1)
##        fvecprint52('uc:',asc(1:end),f1)
#        fvecprint52('uc:',asc(1:end),f1)
        fvecprint52('rt:',time[i][:],f1)
#2        print time
#        fvecprint52('dr:',(rise[i][:]),f1)
        fvecprint52('dr:',rise[i][:],f1)
        fvecprint52('ss:',sv[i][:],f1)
        fvecprint52('ds:',dv[i][:],f1)
#        fvecprint52('ta:','0.225',f1)
        fvecprint52('ta:',rise[i][:]/5.5,f1)
    
    
    #%% write executable shell-script to run DOT9
#6    f2.write('run.Xslip {0} {1}\n'.format(model,oldmod))
#6    f2.write('run.Xseis {0} {1:5.2f} {2:5.2f} {3}\n'.format(model,fq[1]-0.1,fq[2],ddel))
    
    
    #%% close output files; generate executables
    f1.close()
#6    f2.close()
#    !chmod +x run.SS*
