# Created by Hugo Cruz Jimenez, August 2011, KAUST
import sys
import matplotlib
import matplotlib.cm as cm
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure, show
import numpy
import gampdf
import gampdf2
import wblpdf 
import wwbl
import wwbl2
import normpdf 
from mags import *
from val import *
def stat_hypo(*args):

#%function [hypo_x,hypo_z] = stat_hypo(par,mech)
#%---------------------------------------------------------
#% Script to calculate probabilistic hypocenter positions
#% basing upon the paper of Mai et al.(2005).
#%
#% INPUT
#%        slip - slip values on the fault plane
#%        step - step (in Km) of the slip grid
#%        grid - type of grid representation for the input slip
#%               model
#%        mech - faulting mechanism ['dp' -> general dip-slip]
#%                                  ['dc' -> crustal dip-slip]
#%                                  ['ds' -> subduction dip-slip]
#%                                  ['ss' -> strike-slip]
#%                                  ['al' -> global]
#%        Mw   - moment magnitude 
#%
#% OUTPUT
#%        [hypo_x,hypo_z] - hypocenter coordinates on the fault
#%                          plane in km
#%
#% *****
#% This script allows the user to interactively select one or more
#% reasonable hypocenters. Probabilities for location on the entire 
#% plane are depicted (normalized to 1), accordingly to the
#% relationships described in Mai et al.(2005).
#%
#% Author: Walter Imperatori, June 2008
#%---------------------------------------------------------
 
    flag='n'  #%temporary flag for other constraints
#    size=len(args)

    par=args[0]
    mech=args[1]
    
    #% fault grid-dimensions
    dim=numpy.shape(par['slip'])
    print 'DIM', dim
#    dim_z=par['slip'][0]
#    dim_x=par['slip'][1]
#    print 'dim_z, dim_x', dim_z, dim_x
    dim_z=dim[0]
    dim_x=dim[1]
    print 'dim_z, dim_x', dim_z, dim_x
    
    step= par['samp'][0]   #% !samp assumed to be isotropic !
    print 'STEP',step
    
    grid= par['grd']
    dw = par['Mw']
    
    slip= par['slip']
    
    #%if (grid == 'sub')
    
       
     
       #% fault physical dimensions (in km)
    #%   strike_len=(dim_x)*step
    #%   dip_len=(dim_z)*step
    
       #% creates plot axis (in km)
    #%   x=[step/2:step:(strike_len-step/2)]
    #%   y=[step/2:step:(dip_len-step/2)]
    
    #%elif (grid == 'nod')
    
       #% fault physical dimensions (in km)
    #%   strike_len=(dim_x-1)*step
    #%   dip_len=(dim_z-1)*step
    
       #% creates plot axis (in km)
    #%   x=[0:step:strike_len]
    #%   y=[0:step:dip_len]
    #%end
    
    #%-------------------------------------------------------
    #% computes pdf for on-plane position
    
    #% --> downdip

    if mech == 'dp':
        print 'AA',len(dim_z), dim_z
#        hypz=gampdf.gampdf(numpy.linspace(0,1,len(dim_z)),7.364,0.072)
        hypz=gampdf.gampdf(numpy.linspace(0,1,dim_z),7.364,0.072)
    elif mech == 'dc':
        print 'BB',len(dim_z), dim_z
#        hypz=wblpdf.wblpdf(numpy.linspace(0,1,len(dim_z)),0.692,3.394)
        hypz=wblpdf.wblpdf(numpy.linspace(0,1,dim_z),0.692,3.394)
    elif mech == 'ds':
        print 'CC',len(dim_z), dim_z
#        hypz=gampdf.gampdf(numpy.linspace(0,1,len(dim_z)),12.658,0.034)
        hypz=gampdf.gampdf(numpy.linspace(0,1,dim_z),12.658,0.034)
    elif mech == 'ss':
        hypz=wblpdf.wblpdf(numpy.linspace(0,1,dim_z),0.626,3.921)
#        print '  *&*&*&*&  '
#        print 'TIPO', type(hypz) 
        print 'Z1 FORMA HYPZ', numpy.shape(hypz) 
    elif mech == 'al':
#        hypz=wblpdf.wblpdf(numpy.linspace(0,1,len(dim_z)),0.612,3.353)
        hypz=wblpdf.wblpdf(numpy.linspace(0,1,dim_z),0.612,3.353)
    
    #% --> along strike
    hypx=normpdf.normpdf(numpy.linspace(0,1,dim_x),0.5,0.23)
#    print 'X1 FORMA HYPX',numpy.shape(hypx)
    hypx=hypx.reshape(1,len(hypx))
    
    #% calculates and scales to 1 probability based on on-plane position
    print 'hypz ^^^^'
    print '**AAAA FORMAS  hypz, hypx', np.shape(hypz), np.shape(hypx)
#    hypz=hypz.reshape(1,len(hypz)) # Transformada a Transpuesta
    hypz=hypz.reshape(len(hypz),1) # 2  Transformada a Transpuesta
    print 'FORMA TRANSPUESTA', numpy.shape(hypz) 
    hyp_pos=hypz*hypx
    print 'FORMA DE HYP_POS', numpy.shape(hyp_pos) 
    hyp_pos=hyp_pos/numpy.max(hyp_pos)

    print 'formas de hypz, hypx, hyp_pos'
    print numpy.shape(hypz), numpy.shape(hypx), numpy.shape(hyp_pos)
    print type(hypz), type(hypx), type(hyp_pos)

    print hypz
    print hypx
    print hyp_pos
    
    #%------------------------------------------------------
    #% computes pdf based on maximum and mean slip ratio
    
    #% --> computes maximum and mean slip values
    notzero_z,notzero_x=findgt(slip, 0.0)
#    print numpy.shape(notzero_z),numpy.shape(notzero_x)
    for i in range(0,len(notzero_z)):             #%exctracts where slip is not zero
#        print notzero_z
#        ,notzero_x)
#        notzero_slip[i]=slip(0,0)
#        notzero_slip[i]=slip[notzero_z[i]][notzero_x[i]]
        notzero_slip=slip[notzero_z[i]][notzero_x[i]]
#        notzero_slip[i]=slip(notzero_z[i],notzero_x[i])
    mean_slip=numpy.mean(notzero_slip)
    max_slip=numpy.max(slip)
    
    #% --> computes pdf for mean slip ratio
    tmp_slip=slip/mean_slip
    mm,nn=numpy.shape(tmp_slip)
    tmp_slip=tmp_slip.reshape((1,mm*nn))
#    print 'tmp_slip:'
#    print tmp_slip
    if  mech == 'dp' or mech == 'dc' or mech == 'ds':
        hyp_mean=gampdf.gampdf(tmp_slip,2.616,0.623)
        hyp_mean=hyp_mean.reshape(mm,nn)
    elif mech == 'ss':
        hyp_mean=gampdf2.gampdf(tmp_slip,1.928,0.868)
#        hyp_mean=gampdf.gampdf(tmp_slip,1.928,0.868)

        print 'shape of hyp_mean'
        hyp_mean=hyp_mean.reshape(1,len(hyp_mean))
        bbb1=numpy.shape(hyp_mean)
        print bbb1
#       hyp_mean=hyp_mean.reshape(mm,nn)
    elif mech == 'al':
        hyp_mean=gampdf.gampdf(tmp_slip,2.210,0.748)
        hyp_mean=hyp_mean.reshape(mm,nn)
    
#    hyp_mean=hyp_mean.reshape((mm,nn))

    #% --> computes pdf for maximum slip ratio
    tmp_slip=slip/max_slip
#    mm,nn=numpy.shape(tmp_slip)
#    tmp_slip=tmp_slip.reshape((1,mm*nn))
    if mech == 'dp' or mech == 'dc' or mech == 'ds':
        hyp_max=wblpdf.wblpdf(tmp_slip,0.454,1.874)
#        hyp_max=hyp_max.reshape((mm,nn))
    elif  mech == 'ss':
        hyp_max=wwbl2.wblpdf(tmp_slip,0.446,1.551)
        hyp_max=hyp_max.reshape(1,len(hyp_max))
#        hyp_max=wwbl.wblpdf(tmp_slip,0.446,1.551)
##        hyp_max=wblpdf.wblpdf(tmp_slip,0.446,1.551)
#        ooo=shape(hyp_max)
#        print ooo
#        hyp_max=hyp_max.reshape((mm,nn))
        print 'shape of hyp_max'
#       aaa1,bbb1=numpy.shape(hyp_mean)
        bbb1=numpy.shape(hyp_max)
#       print aaa1, bbb1
        print bbb1
    elif  mech == 'al':
        hyp_max=wblpdf.wblpdf(tmp_slip,0.450,1.688)
#        hyp_max=hyp_max.reshape((mm,nn))
    
#    hyp_max=hyp_max.reshape((mm,nn))

    #% --> computes probability based on both previous slip ratios and scales to 1
    print type (hyp_max), type (hyp_mean)
    print ('Arriba hyp_max, hyp_mean')
    hyp_slip=hyp_max*hyp_mean
    print '0000 ****   1    Shape of hyp_slip',numpy.shape(hyp_slip)
    print '1111 ****   2    Shape of hyp_slip',numpy.shape((hyp_slip))
    hyp_slip=hyp_slip/numpy.max(hyp_slip)
      
    print 'shape of hyp_slip, hyp_pos, mm,nn, mm*nn'
    print  numpy.shape(hyp_slip), numpy.shape(hyp_pos), mm, nn, mm*nn
    #% -------------------------------------------------------
    #% computes probability based on both on-plane position and slip ratios
    slip_pos=hyp_slip*hyp_pos  
    slip_pos=slip_pos/numpy.max(slip_pos)   #% scales it to 1 
    
    #% -------------------------------------------------------
    #% define asperities regions (definitions from Mai et al. 2005)
    out_asp_dip,out_asp_strike=findlt(slip, 0.33*max_slip)
    asp_dip,asp_strike=findms(slip, max_slip)
    big_asp_dip,big_asp_strike=findge(slip, 0.66*max_slip)
    
    #% defines asperities positions on the fault plane (0=no asperity, 1=asperity, 2=big asperity)
#    asp_fault(1:dim_z,1:dim_x)=0
    asp_fault=numpy.zeros(dim_z,dim_x)
    for i in range(0,len(asp_dip)):
        asp_fault[asp_dip[i],asp_strike[i]]=1
    for i in range(0,len(big_asp_dip)):
        asp_fault[big_asp_dip[i],big_asp_strike[i]]=2
    
    #% -------------------------------------------------------------------------------------------
    #% applies other constraints (paragraph 'Distance between Hypocenter and Closest Asperities' of
    #% Mai et al. (2005)) --------> to be improved!
    
    
    
    #% --> refreshes hypocenter locations probability according to position inside/outside asperities
    for i in range(0,len(out_asp_dip)):
        slip_pos[out_asp_dip[i],out_asp_strike[i]]=slip_pos[out_asp_dip[i],out_asp_strike[i]]*0.48
    #%    mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.48
    for i in range(0,len(asp_dip)):
        slip_pos[asp_dip[i],asp_strike[i]]=slip_pos[asp_dip[i],asp_strike[i]]*0.35
    #%    mask(asp_dip(i),asp_strike(i))=mask(asp_dip(i),asp_strike(i))*0.35
    for i in range(0,len(big_asp_dip)):
        slip_pos[big_asp_dip[i],big_asp_strike[i]]=slip_pos[big_asp_dip[i],big_asp_strike[i]]*0.16
    #%    mask(big_asp_dip(i),big_asp_strike(i))=mask(big_asp_dip(i),big_asp_strike(i))*0.16
    
    slip_pos=slip_pos/numpy.max(slip_pos)   #% scales to 1
    
    
    if flag == 'y':
    
        #% --> calculates R_max in Km (on the entire fault plane, also where slip=0)
        for i in range(0,dim_z):
            for j in range(0,dim_x):
                for k in range(0,len(notzero_z)):
                    euclid[k]=sqrt(((notzero_z[k]-1)*step-(i)*step)**2 + ((notzero_x[k]-1)*step-(j)*step)**2)
#                    euclid[k]=sqrt(((notzero_z[k]-1)*step-(i-1)*step)**2 + ((notzero_x[k]-1)*step-(j-1)*step)**2)
                R_max[i,j]=numpy.max(euclid)
        
        #% --> defines peak slip and distances to it in Km (on the entire fault plane, also where slip=0)
        tmp_peak_z,tmp_peak_x=findet(slip, numpy.max(slip))
        peak_z=tmp_peak_z[0]   #%takes first maximum if there are more than one
        peak_x=tmp_peak_x[0]
        for i in range(0,dim_z):
            for j in range(0,dim_x):
                peak_dist[i,j]=sqrt(((peak_z-1)*step-(i)*step)**2 + ((peak_x-1)*step-(j)*step)**2)
#                peak_dist[i,j]=sqrt(((peak_z-1)*step-(i-1)*step)**2 + ((peak_x-1)*step-(j-1)*step)**2)
        
        #% --> calculates shortest distance to asperities in Km (only for points outside them)
        for i in range(0,len(out_asp_dip)):
            for j in range(0,len(asp_dip)):
                eul[j]=sqrt(((out_asp_dip[i]-1)*step-(asp_dip[j]-1)*step)**2 + ((out_asp_strike[i]-1)*step-(asp_strike[j]-1)*step)**2)
            asp_dist[out_asp_dip[i],out_asp_strike[i]]=numpy.min(eul)
        
        #% --> calculates shortest distance to big asperities in Km (only for points outside them)
        for i in range(0,len(out_asp_dip)):
            for j in range(0,len(big_asp_dip)):
                eul2[j]=sqrt(((out_asp_dip[i]-1)*step-(big_asp_dip[j]-1)*step)**2 + ((out_asp_strike[i]-1)*step-(big_asp_strike[j]-1)*step)**2)
            big_asp_dist[out_asp_dip[i],out_asp_strike[i]]=numpy.min(eul2)
#&&        for i in range(0,size(asp_dip)):
        for i in range(0,len(asp_dip)):
#&&            for j in range(0,size(big_asp_dip)):
            for j in range(0,len(big_asp_dip)):
                eul3[j]=sqrt(((asp_dip[i]-1)*step-(big_asp_dip[j]-1)*step)**2 + ((asp_strike[i]-1)*step-(big_asp_strike[j]-1)*step)**2)
            big_asp_dist[asp_dip[i],asp_strike[i]]=min(eul3)
        
        mask=numpy.ones(dim_z,dim_x)   #%initialize mask
        
        #% --> applies constraint valid for all hypocenters (on the entire fault) respect to R_max
        tap_z, tap_x=findmd(peak_dist,R_max)
        
        for i in range(0,len(tap_z)):
        #%   slip_pos(tap_z(i),tap_x(i))=0.0
            mask[tap_z[i],tap_x[i]]=0.0
        
        #% --> constraint for hypocenters outside asperities
        for i in range(0,len(out_asp_dip)):
            if asp_dist[out_asp_dip[i],out_asp_strike[i]] <= 0.15*R_max[out_asp_dip[i],out_asp_strike[i]]:
        #%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.95
                mask[out_asp_dip[i],out_asp_strike[i]]=mask[out_asp_dip[i],out_asp_strike[i]]*0.95
            if asp_dist[out_asp_dip[i],out_asp_strike[i]] > 0.15*R_max[out_asp_dip[i],out_asp_strike[i]] and asp_dist[out_asp_dip[i],out_asp_strike[i]] <= 0.3*R_max[out_asp_dip[i],out_asp_strike[i]]:
        #%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.05
                mask[out_asp_dip[i],out_asp_strike[i]]=mask[out_asp_dip[i],out_asp_strike[i]]*0.05
            if asp_dist[out_asp_dip[i],out_asp_strike[i]] > 0.3*R_max[out_asp_dip[i],out_asp_strike[i]]:
        #%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.0
                mask[out_asp_dip[i],out_asp_strike[i]]=mask[out_asp_dip[i],out_asp_strike[i]]*0.0
        
        #% --> constraint for hypocenters outside big asperities
        for i in range(0,len(out_asp_dip)):
            if big_asp_dist[out_asp_dip[i],out_asp_strike[i]] <= 0.4*R_max[out_asp_dip[i],out_asp_strike[i]]:
        #%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.95
                mask[out_asp_dip[i],out_asp_strike[i]]=mask[out_asp_dip[i],out_asp_strike[i]]*0.95
            if big_asp_dist[out_asp_dip[i],out_asp_strike[i]] > 0.4*R_max[out_asp_dip[i],out_asp_strike[i]] and big_asp_dist[out_asp_dip[i],out_asp_strike[i]] <= 0.6*R_max[out_asp_dip[i],out_asp_strike[i]]:
        #%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.05
                mask[out_asp_dip[i],out_asp_strike[i]]=mask[out_asp_dip[i],out_asp_strike[i]]*0.05
            if big_asp_dist[out_asp_dip[i],out_asp_strike[i]] > 0.6*R_max[out_asp_dip[i],out_asp_strike[i]]:
        #%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.0
                mask[out_asp_dip[i],out_asp_strike[i]]=mask[out_asp_dip[i],out_asp_strike[i]]*0.0
        for i in range(0,len(asp_dip)):
            if big_asp_dist[asp_dip[i],asp_strike[i]] <= 0.4*R_max[asp_dip[i],asp_strike[i]]:
        #%      slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.95
                mask[asp_dip[i],asp_strike[i]]=mask[asp_dip[i],asp_strike[i]]*0.95
            if big_asp_dist[asp_dip[i],asp_strike[i]] > 0.4*R_max[asp_dip[i],asp_strike[i]] and big_asp_dist[asp_dip[i],asp_strike[i]] <= 0.6*R_max[asp_dip[i],asp_strike[i]]:
        #%      slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.05
                mask[asp_dip[i],asp_strike[i]]=mask[asp_dip[i],asp_strike[i]]*0.05
            if big_asp_dist[asp_dip[i],asp_strike[i]] > 0.6*R_max[asp_dip[i],asp_strike[i]]:
        #%      slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.0
                mask[asp_dip[i],asp_strike[i]]=mask[asp_dip[i],asp_strike[i]]*0.0
        
        mask=mask/numpy.max(mask)
        #%figurecontour(x,y,mask,[0.2 0.4 0.6 0.8 1],'k')
        #%figureimagesc(x,y,mask)
        
        #% applies mask
        slip_pos=slip_pos*mask
        
        #%normalize to 1
        slip_pos=slip_pos/numpy.max(slip_pos)
    
    x=par['lx']
    y=par['lz']

    return hypo_x,hypo_z

###    
###    #% plot figures
###    #%figureimagesc(x,y,peak_dist);figure,imagesc(x,y,R_max);
    plt.figure()

    ax=plt.subplot(3,1,1)
    im1=plt.imshow(hyp_pos, interpolation='bilinear', origin='upper', cmap=cm.jet, extent=(0,L,W,0) )
#    CS = plt.contour(x, y, hyp_pos, 9, colors='k', origin='image', extent=(0,L,W,0) )    
    CBI = plt.colorbar(im1, orientation='horizontal', shrink=0.8)
###    #%subplot(2,3,2), imagesc(x,y,hyp_mean), colorbar('horiz') subplot(2,3,3), imagesc(x,y,hyp_max), colorbar('horiz');
    ax=plt.subplot(3,1,2)
    im1=plt.imshow(hyp_slip, interpolation='bilinear', origin='upper', cmap=cm.jet, extent=(0,L,W,0) )
#    CS = plt.contour(x, y, hyp_slip, 9, colors='k', origin='image', extent=(0,L,W,0) )    
    CBI = plt.colorbar(im1, orientation='horizontal', shrink=0.8)

    ax=plt.subplot(3,1,3)
    im1=plt.imshow(slip_pos, interpolation='bilinear', origin='upper', cmap=cm.jet, extent=(0,L,W,0) )
    CS = plt.contour(x, y, asp_fault, 9, colors='k', origin='image', extent=(0,L,W,0) )    
    CBI = plt.colorbar(im1, orientation='horizontal', shrink=0.8)

###    subplot(3,1,2), imagesc(x,y,hyp_slip),colorbar('horiz')
###    subplot(3,1,3), imagesc(x,y,slip_pos),colorbar('horiz'), hold on, 
###    contour(x,y,asp_fault,[1 2],'w')
###    #%subplot(2,3,6),imagesc(x,y,slip_pos),colorbar('horiz')hold on;plot(hypo_x,hypo_z,'p','MarkerSize',10,'MarkerFaceColor','w')
###    
###    figure
###    subplot(2,1,1),imagesc(x,y,slip),colorbar('horiz')
###    subplot(2,1,2),imagesc(x,y,slip_pos),colorbar('horiz')
###    
###    
###    
###    find reasonable hypocenters interactively
###    [hypo_x,hypo_z]=getpts
###    
###    subplot(2,1,2),imagesc(x,y,slip_pos),colorbar('horiz') hold on; plot(hypo_x,hypo_z,'p','MarkerSize',12,'MarkerFaceColor','w')
###    subplot(2,1,1),imagesc(x,y,slip),colorbar('horiz') hold on; plot(hypo_x,hypo_z,'p','MarkerSize',12,'MarkerFaceColor','w')
###    
###    
###    
###    
###    
###    #% sort out probability values in a descend fashion
###    #%max_prob=sort(max(slip_pos),'descend')
###    
###    #%size_hyp=(dim_z*dim_x)    %total vectorial dimensions
###    
###    #% ----------------------------------------------------------------------------------------
###    #% different methods to find out reasonable hypocenter locations
###    #%--- first method: highest absolute value
###    #%count=0 flag=0;
###    #%for k=1:size_hyp
###    #%  [tmp_z,tmp_x]=find(slip_pos == max_prob(k))
###    #%  for i=1:size(tmp_z)       %loop effectively performed only if more points have the same value
###    #%     if (peak_dist(tmp_z(i),tmp_x(i)) > 0.04*R_max(tmp_z(i),tmp_x(i)))   %condition valid for all models
###    #%        count=count+1
###    #%        flag=1
###    #%        tmp2_z(count)=tmp_z(i)
###    #%        tmp2_x(count)=tmp_x(i)
###    #%     end
###    #%  end
###    #%  if (flag == 1) 
###    #%     break
###    #%  end
###    #%end
###    #% if there are more candidates, one of them is randomly chosen
###    #%if (count > 1) then
###    #%   i=ceil(unifrnd(0,count))
###    #%end
###    #%hypo_x=(tmp2_x(i)-1)*step hypo_z=(tmp2_z(i)-1)*step;
###    #%--- second method: random selection above a given threshold 
###    
###    #%for i=1:size_hyp
###    #%   [cand_z,cand_x]=find(slip_pos,max_prob(i))
###    #%end
###    #% ---------------------------------------------------------------------------------------------
###
###
####    return hypo_x,hypo_z
