# Created by Hugo Cruz Jimenez, August 2011, KAUST
import sys
import numpy as np
import pickle
from SXSLIPinput2C import *
from RupTime import *
from SSlipReal import *
#from SSlipRealF import *


def aCreateRupt(*args):
 
#function [slip,time,rise,par] = CreateRupt(srcpar,mech,corr,model)
#
# wrapper script or driver to create rupture models for the
# ESC-training course; uses a large number of functions
# collected in ./rupt
#
# INPUT: srcpar = array of source parameters [L W Mw];
#        mech   = 'ss' or 'ds' for strike-slip or dip-slip
#        corr   = array of correlation length [ax az H]
#        model  = 'your_model_name', string to label your model
#
# OUTPUT:slip   = 2D-array of slip-values on the fault
#        time   = 2D-array of rupture onset times
#        rise   = 2D-array of rise times;
#        par    = structural array with source-parameter info
#
# This script also creates the slip-input file for COMPSYN,
# denotes 'model'.SLD where 'model' is the string you give to
# define your model (like 'crazy_rupture')
#
# Martin Mai, AUG 2006
# --------------------

    if len(args)==0:
   
     
        print('=====================================================')
        print('This simple driver-routine will allow you to generate')
        print('random-slip functions, based on some generell source ')
        print('parameters you specify, and prompts you for additional')
        print('parameters on the hypocenter position and the rupture ')
        print('evolution in time (i.e. the ratio of rupture velocity')
        print('to shear-wave velocity, as well as the rise time.')
        print('The code then assembles the full rupture model and')
        print('writes the input file for the XSLIP-routine of the')
        print('COMPSYN-package.')
        print('---------')
        print('The call to this function is as follows:')
        print('..')
        print('[slip,time,rise,par] = CreateRupt(srcpar,mech,corr,model)') 
        print('..')
        print('i.e. the user specifies parameters SRCPAR, MECH, CORR, MODEL')
        print('..')
        print('SRCPAR -- give array [W L Mw]')
        print('MECH   -- ''ss'' for strike-slip, ''ds'' for dip-slip')
        print('CORR   -- array of correlation parameters [ax az H]')
        print('MODEL  -- your favorite model-name string')
        print('---------')
        print('With a little experience you can always simply call the')
        print('individual functions listed here in your MATLAB window')
        print('to be more flexible in what you generate and which models')
        print('you want to use to actually compute ground-motions')
        print('---------')
        print('Once you have run this program, you should have an input')
        print('rupture-model file in your current working directory called')
        print('MODEL.SLD (where MODEL is the string you provide)')
        print('---------')
        print('In a UNIX-shell you then type the following command to execute')
        print('the calculation of synthetic seismograms at ~35 observers:')
        print('"run.model MODEL", writing the seismograms into the directory')
        print('./synthetics in your current directory. There are a number of')
        print('routines to look at your seismograms and to analyze the results')
        print('--> HAVE FUN  <--')
        print('                                 Martin Mai, AUG 2006')
        print('=====================================================')
        
        slip = {}; rise = {}; time = {}; par = {}
        return
   
   
    if len(args)>0 and len(args)< 4:
        print('++++++++++++++++++++++++++++++++++++++++++++++++++++++')
        print('You need to specify some input parameters: ')
        print('SRCPAR, MECH, CORR, MODEL')
        print('++++++++++++++++++++++++++++++++++++++++++++++++++++++')
        print ('Error *** TRY AGAIN !***' )
        return
    elif len(args)==4:
        srcpar=args[0]
        mech=args[1]
        corr=args[2]
        model=args[3]
    else :
        srcpar=args[0]
        mech=args[1]
        corr=args[2]
        model=args[3]
        seed=args[4]

 #%% set some default parameters
 
 
    lines=open('vmodel.dat').readlines()

    vprof=[]

    for line in lines:
        if not(line.startswith('%')):
            vprof.append([ float(field) for field in line.rstrip().rstrip(',').split(',')])

    vprof=np.array(vprof)



    acf=sys.argv[6]

    sa1=float(sys.argv[11])
    sa2=float(sys.argv[12])
    samp = [ sa1 , sa2 ]
    grd=sys.argv[13]
    nexp=float(sys.argv[14])
    wlevel=float(sys.argv[15])
    tp1=float(sys.argv[16])
    tp2=float(sys.argv[17])
    tp3=float(sys.argv[18])
    tp4=float(sys.argv[19])
    taper = [ tp1 , tp2 , tp3, tp4 ]
    depth=float(sys.argv[20])
    dip=float(sys.argv[21])
    fig=sys.argv[22]
    outfile=sys.argv[23]
    
    fa=float(sys.argv[24])
    fb=float(sys.argv[25])
    freq  = [0, fa, fb]
    dur=float(sys.argv[26])
#   print 'Values of frequencies f1, f2', fa, fb
#   print 'This is the value of rise time', dur
    htop=float(sys.argv[27])
    rake=float(sys.argv[28])
    STF=sys.argv[29]
    zmin=float(sys.argv[30])
    
    if mech == 'ss':
        dip = 90.0;  rake = 180.0
    elif mech == 'ds':
        dip = 50.0;  rake = 90.0
     
     #%% parameters for the slip-model generation
     # below the total function call; we fix a few things for simplicity
     # [S,par] = SlipReal(srcpar,mech,acf,corr,seed,samp,grd,...
     #                    nexp,wlevel,taper,depth,dip,fig,outfile)
     #%%%
    print "OUTFILE",outfile 
    slip,par,per=SSlipReal(srcpar,mech,acf,corr,seed,samp,grd,nexp,wlevel,taper,depth,dip,fig,outfile)

    fhh=open('perS','w')
    fhh.write(str(per))
    fhh.close()
    fSS=open('parSS','w')
    fSS.write(str(par))
    fSS.close()

    zmax = zmin+float(srcpar[0])*np.sin(dip/180.0*np.pi)
    xmin = -float(srcpar[1])/2.0

    Hx=float(sys.argv[31])
    Hz=float(sys.argv[32])
    hypo=[ Hx , Hz ]

    print 'hypo', hypo
    slpdir=sys.argv[33]
    vrat=float(sys.argv[34])
    print 'This is vrat', vrat
    vrnd=float(sys.argv[35])

    time = RupTime(slip,samp,hypo,vprof,vrat,vrnd,[zmin, zmax],'n')

    rise = dur*np.ones(np.shape(slip))
#Q #%% update structural-array par
    par['model'] = model
    par['dur'] = dur
    par['STF'] = STF
    par['vprof'] = vprof
    par['htop'] = htop
    par['freq'] = freq
    par['rake'] = rake
    par['vrat'] = vrat
    par['vrnd'] = vrnd
    par['hypo'] = hypo
    par['xmin'] = xmin

#Q #%% handle cases for right-lateral slip or thrust faulting
    if slpdir == 'rl' or slpdir == 'RL' or slpdir == 'tf' or slpdir == 'TF':
        slip = (-1)*slip


    if mech == 'ss':
        SXSLIPinput2C(slip,np.zeros((np.shape(slip))),time,rise,par,'PRUss')
    elif mech == 'ds':
        SXSLIPinput2C(np.zeros((np.shape(slip))),slip,time,rise,par,'PRUds')
